@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2018 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) Zend Technologies Ltd. (http://www.zend.com)           |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,12 +12,10 @@
    | obtain it through the world-wide-web, please send a note to          |
    | license@zend.com so we can mail you a copy immediately.              |
    +----------------------------------------------------------------------+
-   | Author: Zeev Suraski <zeev@zend.com>                                 |
+   | Author: Zeev Suraski <zeev@php.net>                                  |
    +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
-
 #include "zend.h"
 #include "zend_sort.h"
 #include "zend_API.h"
@@ -25,6 +23,7 @@
 #include "zend_alloc.h"
 #include "zend_operators.h"
 #include "zend_strtod.h"
+#include "zend_modules.h"
 
 static HashTable *registered_zend_ini_directives;
 
@@ -38,17 +37,14 @@ static int zend_remove_ini_entries(zval *el, void *arg) /* {{{ */
 {
 	zend_ini_entry *ini_entry = (zend_ini_entry *)Z_PTR_P(el);
 	int module_number = *(int *)arg;
-	if (ini_entry->module_number == module_number) {
-		return 1;
-	} else {
-		return 0;
-	}
+
+	return ini_entry->module_number == module_number;
 }
 /* }}} */
 
-static int zend_restore_ini_entry_cb(zend_ini_entry *ini_entry, int stage) /* {{{ */
+static zend_result zend_restore_ini_entry_cb(zend_ini_entry *ini_entry, int stage) /* {{{ */
 {
-	int result = FAILURE;
+	zend_result result = FAILURE;
 
 	if (ini_entry->modified) {
 		if (ini_entry->on_modify) {
@@ -61,7 +57,7 @@ static int zend_restore_ini_entry_cb(zend_ini_entry *ini_entry, int stage) /* {{
 		}
 		if (stage == ZEND_INI_STAGE_RUNTIME && result == FAILURE) {
 			/* runtime failure is OK */
-			return 1;
+			return FAILURE;
 		}
 		if (ini_entry->value != ini_entry->orig_value) {
 			zend_string_release(ini_entry->value);
@@ -72,15 +68,7 @@ static int zend_restore_ini_entry_cb(zend_ini_entry *ini_entry, int stage) /* {{
 		ini_entry->orig_value = NULL;
 		ini_entry->orig_modifiable = 0;
 	}
-	return 0;
-}
-/* }}} */
-
-static int zend_restore_ini_entry_wrapper(zval *el) /* {{{ */
-{
-	zend_ini_entry *ini_entry = (zend_ini_entry *)Z_PTR_P(el);
-	zend_restore_ini_entry_cb(ini_entry, ZEND_INI_STAGE_DEACTIVATE);
-	return 1;
+	return SUCCESS;
 }
 /* }}} */
 
@@ -88,12 +76,12 @@ static void free_ini_entry(zval *zv) /* {{{ */
 {
 	zend_ini_entry *entry = (zend_ini_entry*)Z_PTR_P(zv);
 
-	zend_string_release(entry->name);
+	zend_string_release_ex(entry->name, 1);
 	if (entry->value) {
 		zend_string_release(entry->value);
 	}
 	if (entry->orig_value) {
-		zend_string_release(entry->orig_value);
+		zend_string_release_ex(entry->orig_value, 1);
 	}
 	free(entry);
 }
@@ -102,22 +90,20 @@ static void free_ini_entry(zval *zv) /* {{{ */
 /*
  * Startup / shutdown
  */
-ZEND_API int zend_ini_startup(void) /* {{{ */
+ZEND_API void zend_ini_startup(void) /* {{{ */
 {
 	registered_zend_ini_directives = (HashTable *) malloc(sizeof(HashTable));
 
 	EG(ini_directives) = registered_zend_ini_directives;
 	EG(modified_ini_directives) = NULL;
 	EG(error_reporting_ini_entry) = NULL;
-	zend_hash_init_ex(registered_zend_ini_directives, 128, NULL, free_ini_entry, 1, 0);
-	return SUCCESS;
+	zend_hash_init(registered_zend_ini_directives, 128, NULL, free_ini_entry, 1);
 }
 /* }}} */
 
-ZEND_API int zend_ini_shutdown(void) /* {{{ */
+ZEND_API void zend_ini_shutdown(void) /* {{{ */
 {
 	zend_ini_dtor(EG(ini_directives));
-	return SUCCESS;
 }
 /* }}} */
 
@@ -128,23 +114,25 @@ ZEND_API void zend_ini_dtor(HashTable *ini_directives) /* {{{ */
 }
 /* }}} */
 
-ZEND_API int zend_ini_global_shutdown(void) /* {{{ */
+ZEND_API void zend_ini_global_shutdown(void) /* {{{ */
 {
 	zend_hash_destroy(registered_zend_ini_directives);
 	free(registered_zend_ini_directives);
-	return SUCCESS;
 }
 /* }}} */
 
-ZEND_API int zend_ini_deactivate(void) /* {{{ */
+ZEND_API void zend_ini_deactivate(void) /* {{{ */
 {
 	if (EG(modified_ini_directives)) {
-		zend_hash_apply(EG(modified_ini_directives), zend_restore_ini_entry_wrapper);
+		zend_ini_entry *ini_entry;
+
+		ZEND_HASH_FOREACH_PTR(EG(modified_ini_directives), ini_entry) {
+			zend_restore_ini_entry_cb(ini_entry, ZEND_INI_STAGE_DEACTIVATE);
+		} ZEND_HASH_FOREACH_END();
 		zend_hash_destroy(EG(modified_ini_directives));
 		FREE_HASHTABLE(EG(modified_ini_directives));
 		EG(modified_ini_directives) = NULL;
 	}
-	return SUCCESS;
 }
 /* }}} */
 
@@ -157,39 +145,37 @@ static void copy_ini_entry(zval *zv) /* {{{ */
 	Z_PTR_P(zv) = new_entry;
 	memcpy(new_entry, old_entry, sizeof(zend_ini_entry));
 	if (old_entry->name) {
-		new_entry->name = zend_string_init(ZSTR_VAL(old_entry->name), ZSTR_LEN(old_entry->name), 1);
+		new_entry->name = zend_string_dup(old_entry->name, 1);
 	}
 	if (old_entry->value) {
-		new_entry->value = zend_string_init(ZSTR_VAL(old_entry->value), ZSTR_LEN(old_entry->value), 1);
+		new_entry->value = zend_string_dup(old_entry->value, 1);
 	}
 	if (old_entry->orig_value) {
-		new_entry->orig_value = zend_string_init(ZSTR_VAL(old_entry->orig_value), ZSTR_LEN(old_entry->orig_value), 1);
+		new_entry->orig_value = zend_string_dup(old_entry->orig_value, 1);
 	}
 }
 /* }}} */
 
-ZEND_API int zend_copy_ini_directives(void) /* {{{ */
+ZEND_API void zend_copy_ini_directives(void) /* {{{ */
 {
 	EG(modified_ini_directives) = NULL;
 	EG(error_reporting_ini_entry) = NULL;
 	EG(ini_directives) = (HashTable *) malloc(sizeof(HashTable));
-	zend_hash_init_ex(EG(ini_directives), registered_zend_ini_directives->nNumOfElements, NULL, free_ini_entry, 1, 0);
+	zend_hash_init(EG(ini_directives), registered_zend_ini_directives->nNumOfElements, NULL, free_ini_entry, 1);
 	zend_hash_copy(EG(ini_directives), registered_zend_ini_directives, copy_ini_entry);
-	return SUCCESS;
 }
 /* }}} */
 #endif
 
-static int ini_key_compare(const void *a, const void *b) /* {{{ */
+static int ini_key_compare(Bucket *f, Bucket *s) /* {{{ */
 {
-	const Bucket *f;
-	const Bucket *s;
-
-	f = (const Bucket *) a;
-	s = (const Bucket *) b;
-
 	if (!f->key && !s->key) { /* both numeric */
-		return ZEND_NORMALIZE_BOOL(f->h - s->h);
+		if (f->h > s->h) {
+			return -1;
+		} else if (f->h < s->h) {
+			return 1;
+		}
+		return 0;
 	} else if (!f->key) { /* f is numeric, s is not */
 		return -1;
 	} else if (!s->key) { /* s is numeric, f is not */
@@ -209,7 +195,7 @@ ZEND_API void zend_ini_sort_entries(void) /* {{{ */
 /*
  * Registration / unregistration
  */
-ZEND_API int zend_register_ini_entries(const zend_ini_entry_def *ini_entry, int module_number) /* {{{ */
+ZEND_API zend_result zend_register_ini_entries_ex(const zend_ini_entry_def *ini_entry, int module_number, int module_type) /* {{{ */
 {
 	zend_ini_entry *p;
 	zval *default_value;
@@ -225,13 +211,16 @@ ZEND_API int zend_register_ini_entries(const zend_ini_entry_def *ini_entry, int
 	 * lead to death.
 	 */
 	if (directives != EG(ini_directives)) {
+		ZEND_ASSERT(module_type == MODULE_TEMPORARY);
 		directives = EG(ini_directives);
+	} else {
+		ZEND_ASSERT(module_type == MODULE_PERSISTENT);
 	}
 #endif
 
 	while (ini_entry->name) {
 		p = pemalloc(sizeof(zend_ini_entry), 1);
-		p->name = zend_string_init(ini_entry->name, ini_entry->name_length, 1);
+		p->name = zend_string_init_interned(ini_entry->name, ini_entry->name_length, 1);
 		p->on_modify = ini_entry->on_modify;
 		p->mh_arg1 = ini_entry->mh_arg1;
 		p->mh_arg2 = ini_entry->mh_arg2;
@@ -247,18 +236,18 @@ ZEND_API int zend_register_ini_entries(const zend_ini_entry_def *ini_entry, int
 
 		if (zend_hash_add_ptr(directives, p->name, (void*)p) == NULL) {
 			if (p->name) {
-				zend_string_release(p->name);
+				zend_string_release_ex(p->name, 1);
 			}
-			zend_unregister_ini_entries(module_number);
+			zend_unregister_ini_entries_ex(module_number, module_type);
 			return FAILURE;
 		}
 		if (((default_value = zend_get_configuration_directive(p->name)) != NULL) &&
-            (!p->on_modify || p->on_modify(p, Z_STR_P(default_value), p->mh_arg1, p->mh_arg2, p->mh_arg3, ZEND_INI_STAGE_STARTUP) == SUCCESS)) {
+		    (!p->on_modify || p->on_modify(p, Z_STR_P(default_value), p->mh_arg1, p->mh_arg2, p->mh_arg3, ZEND_INI_STAGE_STARTUP) == SUCCESS)) {
 
-			p->value = zend_string_copy(Z_STR_P(default_value));
+			p->value = zend_new_interned_string(zend_string_copy(Z_STR_P(default_value)));
 		} else {
 			p->value = ini_entry->value ?
-				zend_string_init(ini_entry->value, ini_entry->value_length, 1) : NULL;
+				zend_string_init_interned(ini_entry->value, ini_entry->value_length, 1) : NULL;
 
 			if (p->on_modify) {
 				p->on_modify(p, p->value, p->mh_arg1, p->mh_arg2, p->mh_arg3, ZEND_INI_STAGE_STARTUP);
@@ -270,69 +259,100 @@ ZEND_API int zend_register_ini_entries(const zend_ini_entry_def *ini_entry, int
 }
 /* }}} */
 
+ZEND_API zend_result zend_register_ini_entries(const zend_ini_entry_def *ini_entry, int module_number) /* {{{ */
+{
+	zend_module_entry *module;
+
+	/* Module is likely to be the last one in the list */
+	ZEND_HASH_REVERSE_FOREACH_PTR(&module_registry, module) {
+		if (module->module_number == module_number) {
+			return zend_register_ini_entries_ex(ini_entry, module_number, module->type);
+		}
+	} ZEND_HASH_FOREACH_END();
+
+	return FAILURE;
+}
+/* }}} */
+
+ZEND_API void zend_unregister_ini_entries_ex(int module_number, int module_type) /* {{{ */
+{
+	static HashTable *ini_directives;
+
+	if (module_type == MODULE_TEMPORARY) {
+		ini_directives = EG(ini_directives);
+	} else {
+		ini_directives = registered_zend_ini_directives;
+	}
+
+	zend_hash_apply_with_argument(ini_directives, zend_remove_ini_entries, (void *) &module_number);
+}
+/* }}} */
+
 ZEND_API void zend_unregister_ini_entries(int module_number) /* {{{ */
 {
-	zend_hash_apply_with_argument(registered_zend_ini_directives, zend_remove_ini_entries, (void *) &module_number);
+	zend_module_entry *module;
+
+	/* Module is likely to be the last one in the list */
+	ZEND_HASH_REVERSE_FOREACH_PTR(&module_registry, module) {
+		if (module->module_number == module_number) {
+			zend_unregister_ini_entries_ex(module_number, module->type);
+			return;
+		}
+	} ZEND_HASH_FOREACH_END();
 }
 /* }}} */
 
 #ifdef ZTS
-static int zend_ini_refresh_cache(zval *el, void *arg) /* {{{ */
-{
-	zend_ini_entry *p = (zend_ini_entry *)Z_PTR_P(el);
-	int stage = (int)(zend_intptr_t)arg;
-
-	if (p->on_modify) {
-		p->on_modify(p, p->value, p->mh_arg1, p->mh_arg2, p->mh_arg3, stage);
-	}
-	return 0;
-}
-/* }}} */
-
 ZEND_API void zend_ini_refresh_caches(int stage) /* {{{ */
 {
-	zend_hash_apply_with_argument(EG(ini_directives), zend_ini_refresh_cache, (void *)(zend_intptr_t) stage);
+	zend_ini_entry *p;
+
+	ZEND_HASH_FOREACH_PTR(EG(ini_directives), p) {
+		if (p->on_modify) {
+			p->on_modify(p, p->value, p->mh_arg1, p->mh_arg2, p->mh_arg3, stage);
+		}
+	} ZEND_HASH_FOREACH_END();
 }
 /* }}} */
 #endif
 
-ZEND_API int zend_alter_ini_entry(zend_string *name, zend_string *new_value, int modify_type, int stage) /* {{{ */
+ZEND_API zend_result zend_alter_ini_entry(zend_string *name, zend_string *new_value, int modify_type, int stage) /* {{{ */
 {
 
 	return zend_alter_ini_entry_ex(name, new_value, modify_type, stage, 0);
 }
 /* }}} */
 
-ZEND_API int zend_alter_ini_entry_chars(zend_string *name, const char *value, size_t value_length, int modify_type, int stage) /* {{{ */
+ZEND_API zend_result zend_alter_ini_entry_chars(zend_string *name, const char *value, size_t value_length, int modify_type, int stage) /* {{{ */
 {
-    int ret;
-    zend_string *new_value;
+	zend_result ret;
+	zend_string *new_value;
 
-	new_value = zend_string_init(value, value_length, stage != ZEND_INI_STAGE_RUNTIME);
+	new_value = zend_string_init(value, value_length, !(stage & ZEND_INI_STAGE_IN_REQUEST));
 	ret = zend_alter_ini_entry_ex(name, new_value, modify_type, stage, 0);
 	zend_string_release(new_value);
 	return ret;
 }
 /* }}} */
 
-ZEND_API int zend_alter_ini_entry_chars_ex(zend_string *name, const char *value, size_t value_length, int modify_type, int stage, int force_change) /* {{{ */
+ZEND_API zend_result zend_alter_ini_entry_chars_ex(zend_string *name, const char *value, size_t value_length, int modify_type, int stage, int force_change) /* {{{ */
 {
-    int ret;
-    zend_string *new_value;
+	zend_result ret;
+	zend_string *new_value;
 
-	new_value = zend_string_init(value, value_length, stage != ZEND_INI_STAGE_RUNTIME);
+	new_value = zend_string_init(value, value_length, !(stage & ZEND_INI_STAGE_IN_REQUEST));
 	ret = zend_alter_ini_entry_ex(name, new_value, modify_type, stage, force_change);
 	zend_string_release(new_value);
 	return ret;
 }
 /* }}} */
 
-ZEND_API int zend_alter_ini_entry_ex(zend_string *name, zend_string *new_value, int modify_type, int stage, int force_change) /* {{{ */
+ZEND_API zend_result zend_alter_ini_entry_ex(zend_string *name, zend_string *new_value, int modify_type, int stage, bool force_change) /* {{{ */
 {
 	zend_ini_entry *ini_entry;
 	zend_string *duplicate;
-	zend_bool modifiable;
-	zend_bool modified;
+	uint8_t modifiable;
+	bool modified;
 
 	if ((ini_entry = zend_hash_find_ptr(EG(ini_directives), name)) == NULL) {
 		return FAILURE;
@@ -379,7 +399,7 @@ ZEND_API int zend_alter_ini_entry_ex(zend_string *name, zend_string *new_value,
 }
 /* }}} */
 
-ZEND_API int zend_restore_ini_entry(zend_string *name, int stage) /* {{{ */
+ZEND_API zend_result zend_restore_ini_entry(zend_string *name, int stage) /* {{{ */
 {
 	zend_ini_entry *ini_entry;
 
@@ -400,7 +420,7 @@ ZEND_API int zend_restore_ini_entry(zend_string *name, int stage) /* {{{ */
 }
 /* }}} */
 
-ZEND_API int zend_ini_register_displayer(char *name, uint32_t name_length, void (*displayer)(zend_ini_entry *ini_entry, int type)) /* {{{ */
+ZEND_API zend_result zend_ini_register_displayer(const char *name, uint32_t name_length, void (*displayer)(zend_ini_entry *ini_entry, int type)) /* {{{ */
 {
 	zend_ini_entry *ini_entry;
 
@@ -418,7 +438,7 @@ ZEND_API int zend_ini_register_displayer(char *name, uint32_t name_length, void
  * Data retrieval
  */
 
-ZEND_API zend_long zend_ini_long(char *name, uint32_t name_length, int orig) /* {{{ */
+ZEND_API zend_long zend_ini_long(const char *name, size_t name_length, int orig) /* {{{ */
 {
 	zend_ini_entry *ini_entry;
 
@@ -435,7 +455,7 @@ ZEND_API zend_long zend_ini_long(char *name, uint32_t name_length, int orig) /*
 }
 /* }}} */
 
-ZEND_API double zend_ini_double(char *name, uint32_t name_length, int orig) /* {{{ */
+ZEND_API double zend_ini_double(const char *name, size_t name_length, int orig) /* {{{ */
 {
 	zend_ini_entry *ini_entry;
 
@@ -452,7 +472,7 @@ ZEND_API double zend_ini_double(char *name, uint32_t name_length, int orig) /* {
 }
 /* }}} */
 
-ZEND_API char *zend_ini_string_ex(char *name, uint32_t name_length, int orig, zend_bool *exists) /* {{{ */
+ZEND_API char *zend_ini_string_ex(const char *name, size_t name_length, int orig, bool *exists) /* {{{ */
 {
 	zend_ini_entry *ini_entry;
 
@@ -476,9 +496,9 @@ ZEND_API char *zend_ini_string_ex(char *name, uint32_t name_length, int orig, ze
 }
 /* }}} */
 
-ZEND_API char *zend_ini_string(char *name, uint32_t name_length, int orig) /* {{{ */
+ZEND_API char *zend_ini_string(const char *name, size_t name_length, int orig) /* {{{ */
 {
-	zend_bool exists = 1;
+	bool exists = 1;
 	char *return_value;
 
 	return_value = zend_ini_string_ex(name, name_length, orig, &exists);
@@ -491,45 +511,30 @@ ZEND_API char *zend_ini_string(char *name, uint32_t name_length, int orig) /* {{
 }
 /* }}} */
 
-#if TONY_20070307
-static void zend_ini_displayer_cb(zend_ini_entry *ini_entry, int type) /* {{{ */
+ZEND_API zend_string *zend_ini_get_value(zend_string *name) /* {{{ */
 {
-	if (ini_entry->displayer) {
-		ini_entry->displayer(ini_entry, type);
-	} else {
-		char *display_string;
-		uint32_t display_string_length;
+	zend_ini_entry *ini_entry;
 
-		if (type == ZEND_INI_DISPLAY_ORIG && ini_entry->modified) {
-			if (ini_entry->orig_value) {
-				display_string = ini_entry->orig_value;
-				display_string_length = ini_entry->orig_value_length;
-			} else {
-				if (zend_uv.html_errors) {
-					display_string = NO_VALUE_HTML;
-					display_string_length = sizeof(NO_VALUE_HTML) - 1;
-				} else {
-					display_string = NO_VALUE_PLAINTEXT;
-					display_string_length = sizeof(NO_VALUE_PLAINTEXT) - 1;
-				}
-			}
-		} else if (ini_entry->value && ini_entry->value[0]) {
-			display_string = ini_entry->value;
-			display_string_length = ini_entry->value_length;
-		} else {
-			if (zend_uv.html_errors) {
-				display_string = NO_VALUE_HTML;
-				display_string_length = sizeof(NO_VALUE_HTML) - 1;
-			} else {
-				display_string = NO_VALUE_PLAINTEXT;
-				display_string_length = sizeof(NO_VALUE_PLAINTEXT) - 1;
-			}
-		}
-		ZEND_WRITE(display_string, display_string_length);
+	ini_entry = zend_hash_find_ptr(EG(ini_directives), name);
+	if (ini_entry) {
+		return ini_entry->value ? ini_entry->value : ZSTR_EMPTY_ALLOC();
+	} else {
+		return NULL;
 	}
 }
 /* }}} */
-#endif
+
+ZEND_API bool zend_ini_parse_bool(zend_string *str)
+{
+	if (zend_string_equals_literal_ci(str, "true")
+			|| zend_string_equals_literal_ci(str, "yes")
+			|| zend_string_equals_literal_ci(str, "on")
+	) {
+		return 1;
+	} else {
+		return atoi(ZSTR_VAL(str)) != 0;
+	}
+}
 
 ZEND_INI_DISP(zend_ini_boolean_displayer_cb) /* {{{ */
 {
@@ -545,15 +550,7 @@ ZEND_INI_DISP(zend_ini_boolean_displayer_cb) /* {{{ */
 	}
 
 	if (tmp_value) {
-		if (ZSTR_LEN(tmp_value) == 4 && strcasecmp(ZSTR_VAL(tmp_value), "true") == 0) {
-			value = 1;
-		} else if (ZSTR_LEN(tmp_value) == 3 && strcasecmp(ZSTR_VAL(tmp_value), "yes") == 0) {
-			value = 1;
-		} else if (ZSTR_LEN(tmp_value) == 2 && strcasecmp(ZSTR_VAL(tmp_value), "on") == 0) {
-			value = 1;
-		} else {
-			value = atoi(ZSTR_VAL(tmp_value));
-		}
+		value = zend_ini_parse_bool(tmp_value);
 	} else {
 		value = 0;
 	}
@@ -618,68 +615,28 @@ ZEND_INI_DISP(display_link_numbers) /* {{{ */
 /* Standard message handlers */
 ZEND_API ZEND_INI_MH(OnUpdateBool) /* {{{ */
 {
-	zend_bool *p;
-#ifndef ZTS
-	char *base = (char *) mh_arg2;
-#else
-	char *base;
-
-	base = (char *) ts_resource(*((int *) mh_arg2));
-#endif
-
-	p = (zend_bool *) (base+(size_t) mh_arg1);
-
-	if (ZSTR_LEN(new_value) == 2 && strcasecmp("on", ZSTR_VAL(new_value)) == 0) {
-		*p = (zend_bool) 1;
-	}
-	else if (ZSTR_LEN(new_value) == 3 && strcasecmp("yes", ZSTR_VAL(new_value)) == 0) {
-		*p = (zend_bool) 1;
-	}
-	else if (ZSTR_LEN(new_value) == 4 && strcasecmp("true", ZSTR_VAL(new_value)) == 0) {
-		*p = (zend_bool) 1;
-	}
-	else {
-		*p = (zend_bool) atoi(ZSTR_VAL(new_value));
-	}
+	bool *p = (bool *) ZEND_INI_GET_ADDR();
+	*p = zend_ini_parse_bool(new_value);
 	return SUCCESS;
 }
 /* }}} */
 
 ZEND_API ZEND_INI_MH(OnUpdateLong) /* {{{ */
 {
-	zend_long *p;
-#ifndef ZTS
-	char *base = (char *) mh_arg2;
-#else
-	char *base;
-
-	base = (char *) ts_resource(*((int *) mh_arg2));
-#endif
-
-	p = (zend_long *) (base+(size_t) mh_arg1);
-
-	*p = zend_atol(ZSTR_VAL(new_value), (int)ZSTR_LEN(new_value));
+	zend_long *p = (zend_long *) ZEND_INI_GET_ADDR();
+	*p = zend_atol(ZSTR_VAL(new_value), ZSTR_LEN(new_value));
 	return SUCCESS;
 }
 /* }}} */
 
 ZEND_API ZEND_INI_MH(OnUpdateLongGEZero) /* {{{ */
 {
-	zend_long *p, tmp;
-#ifndef ZTS
-	char *base = (char *) mh_arg2;
-#else
-	char *base;
-
-	base = (char *) ts_resource(*((int *) mh_arg2));
-#endif
-
-	tmp = zend_atol(ZSTR_VAL(new_value), (int)ZSTR_LEN(new_value));
+	zend_long tmp = zend_atol(ZSTR_VAL(new_value), ZSTR_LEN(new_value));
 	if (tmp < 0) {
 		return FAILURE;
 	}
 
-	p = (zend_long *) (base+(size_t) mh_arg1);
+	zend_long *p = (zend_long *) ZEND_INI_GET_ADDR();
 	*p = tmp;
 
 	return SUCCESS;
@@ -688,17 +645,7 @@ ZEND_API ZEND_INI_MH(OnUpdateLongGEZero) /* {{{ */
 
 ZEND_API ZEND_INI_MH(OnUpdateReal) /* {{{ */
 {
-	double *p;
-#ifndef ZTS
-	char *base = (char *) mh_arg2;
-#else
-	char *base;
-
-	base = (char *) ts_resource(*((int *) mh_arg2));
-#endif
-
-	p = (double *) (base+(size_t) mh_arg1);
-
+	double *p = (double *) ZEND_INI_GET_ADDR();
 	*p = zend_strtod(ZSTR_VAL(new_value), NULL);
 	return SUCCESS;
 }
@@ -706,17 +653,7 @@ ZEND_API ZEND_INI_MH(OnUpdateReal) /* {{{ */
 
 ZEND_API ZEND_INI_MH(OnUpdateString) /* {{{ */
 {
-	char **p;
-#ifndef ZTS
-	char *base = (char *) mh_arg2;
-#else
-	char *base;
-
-	base = (char *) ts_resource(*((int *) mh_arg2));
-#endif
-
-	p = (char **) (base+(size_t) mh_arg1);
-
+	char **p = (char **) ZEND_INI_GET_ADDR();
 	*p = new_value ? ZSTR_VAL(new_value) : NULL;
 	return SUCCESS;
 }
@@ -724,32 +661,12 @@ ZEND_API ZEND_INI_MH(OnUpdateString) /* {{{ */
 
 ZEND_API ZEND_INI_MH(OnUpdateStringUnempty) /* {{{ */
 {
-	char **p;
-#ifndef ZTS
-	char *base = (char *) mh_arg2;
-#else
-	char *base;
-
-	base = (char *) ts_resource(*((int *) mh_arg2));
-#endif
-
 	if (new_value && !ZSTR_VAL(new_value)[0]) {
 		return FAILURE;
 	}
 
-	p = (char **) (base+(size_t) mh_arg1);
-
+	char **p = (char **) ZEND_INI_GET_ADDR();
 	*p = new_value ? ZSTR_VAL(new_value) : NULL;
 	return SUCCESS;
 }
 /* }}} */
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * indent-tabs-mode: t
- * End:
- * vim600: sw=4 ts=4 fdm=marker
- * vim<600: sw=4 ts=4
- */
