@@ -2563,6 +2563,11 @@ static void ZEND_FASTCALL zend_jit_assign_op_to_typed_prop(zval *zptr, zend_prop
 	zend_execute_data *execute_data = EG(current_execute_data);
 	zval z_copy;
 
+	if (UNEXPECTED(prop_info->flags & ZEND_ACC_READONLY)) {
+		zend_readonly_property_modification_error(prop_info);
+		return;
+	}
+
 	ZVAL_DEREF(zptr);
 	/* Make sure that in-place concatenation is used if the LHS is a string. */
 	if (binary_op == concat_function && Z_TYPE_P(zptr) == IS_STRING) {
@@ -2650,6 +2655,13 @@ static ZEND_COLD zend_long _zend_jit_throw_dec_prop_error(zend_property_info *pr
 
 static void ZEND_FASTCALL zend_jit_inc_typed_prop(zval *var_ptr, zend_property_info *prop_info)
 {
+	ZEND_ASSERT(Z_TYPE_P(var_ptr) != IS_UNDEF);
+
+	if (UNEXPECTED((prop_info->flags & ZEND_ACC_READONLY))) {
+		zend_readonly_property_modification_error(prop_info);
+		return;
+	}
+
 	zend_execute_data *execute_data = EG(current_execute_data);
 	zval tmp;
 
@@ -2673,6 +2685,13 @@ static void ZEND_FASTCALL zend_jit_inc_typed_prop(zval *var_ptr, zend_property_i
 
 static void ZEND_FASTCALL zend_jit_dec_typed_prop(zval *var_ptr, zend_property_info *prop_info)
 {
+	ZEND_ASSERT(Z_TYPE_P(var_ptr) != IS_UNDEF);
+
+	if (UNEXPECTED((prop_info->flags & ZEND_ACC_READONLY))) {
+		zend_readonly_property_modification_error(prop_info);
+		return;
+	}
+
 	zend_execute_data *execute_data = EG(current_execute_data);
 	zval tmp;
 
@@ -2710,6 +2729,16 @@ static void ZEND_FASTCALL zend_jit_pre_dec_typed_prop(zval *var_ptr, zend_proper
 
 static void ZEND_FASTCALL zend_jit_post_inc_typed_prop(zval *var_ptr, zend_property_info *prop_info, zval *result)
 {
+	ZEND_ASSERT(Z_TYPE_P(var_ptr) != IS_UNDEF);
+
+	if (UNEXPECTED((prop_info->flags & ZEND_ACC_READONLY))) {
+		zend_readonly_property_modification_error(prop_info);
+		if (result) {
+			ZVAL_UNDEF(result);
+		}
+		return;
+	}
+
 	zend_execute_data *execute_data = EG(current_execute_data);
 
 	ZVAL_DEREF(var_ptr);
@@ -2731,6 +2760,16 @@ static void ZEND_FASTCALL zend_jit_post_inc_typed_prop(zval *var_ptr, zend_prope
 
 static void ZEND_FASTCALL zend_jit_post_dec_typed_prop(zval *var_ptr, zend_property_info *prop_info, zval *result)
 {
+	ZEND_ASSERT(Z_TYPE_P(var_ptr) != IS_UNDEF);
+
+	if (UNEXPECTED((prop_info->flags & ZEND_ACC_READONLY))) {
+		zend_readonly_property_modification_error(prop_info);
+		if (result) {
+			ZVAL_UNDEF(result);
+		}
+		return;
+	}
+
 	zend_execute_data *execute_data = EG(current_execute_data);
 
 	ZVAL_DEREF(var_ptr);
@@ -3030,7 +3069,19 @@ static zend_result ZEND_FASTCALL zval_jit_update_constant_ex(zval *p, zend_class
 		} else {
 			zval tmp;
 
-			if (UNEXPECTED(zend_ast_evaluate(&tmp, ast, scope) != SUCCESS)) {
+			// Increase the refcount during zend_ast_evaluate to avoid releasing the ast too early
+			// on nested calls to zval_update_constant_ex which can happen when retriggering ast
+			// evaluation during autoloading.
+			zend_ast_ref *ast_ref = Z_AST_P(p);
+			bool ast_is_refcounted = !(GC_FLAGS(ast_ref) & GC_IMMUTABLE);
+			if (ast_is_refcounted) {
+				GC_ADDREF(ast_ref);
+			}
+			zend_result result = zend_ast_evaluate(&tmp, ast, scope);
+			if (ast_is_refcounted && !GC_DELREF(ast_ref)) {
+				rc_dtor_func((zend_refcounted *)ast_ref);
+			}
+			if (UNEXPECTED(result != SUCCESS)) {
 				return FAILURE;
 			}
 			zval_ptr_dtor_nogc(p);
